from typing import Any
import customtkinter as ctk
import tkscenes


class SidePane(ctk.CTkBaseClass):
    def __init__(self, master: Any, **kwargs):
        """
        Create a side panel
        :param master:
        :param kwargs:
        """
        super().__init__(master, **kwargs)

        self.pane = ctk.CTkFrame(master)
        self.center = ctk.CTkFrame(self.pane, fg_color="transparent")
        self.center.pack(expand=True, anchor="center")

        self.items: dict[str, ctk.CTkButton] = {}

    def add(self, key: str, name=None, image=None, command=None, anchor="center"):
        """
        Add a entry to side panel
        :param key: Indentifier
        :param name: Name of the entry
        :param image:
        :param command:
        :param anchor:
        :return:
        """
        if name is None:
            name = key

        self.items[key] = ctk.CTkButton(self.center, text=name, image=image, command=command, anchor=anchor)

        self.items[key].pack(padx=5, pady=5)

    def pack(self, **kwargs):
        """
        Pack the side panel and it's entries
        :param kwargs:
        :key padx: Add padding in x direction
        :key pady: Add padding in y direction
        :key ipadx: Add internal padding in x direction
        :key ipady: Add internal padding in x direction
        :return:
        """
        settings = {
            "padx": 0,
            "pady": 0,
            "ipadx": 0,
            "ipady": 0,
            "side": "top",
            "anchor": "center",
            "fill": None,
            "expand": False
        }

        for key in settings.keys():
            if key in kwargs.keys():
                settings[key] = kwargs[key]

        self.pane.pack(
            padx=settings["padx"],
            pady=settings["pady"],
            ipadx=settings["ipadx"],
            ipady=settings["ipady"],
            side=settings["side"],
            anchor=settings["anchor"],
            fill=settings["fill"],
            expand=settings["expand"]
        )

    def pack_forget(self):
        """
        Unmap the side panel
        :return:
        """
        self.pane.pack_forget()


class MainPane(ctk.CTkBaseClass):
    """
    Create a main panel
    """
    def __init__(self, master: Any, fg_color=None, bg_color="transparent", **kwargs):
        super().__init__(master, **kwargs)

        self.pane = ctk.CTkFrame(master, fg_color=fg_color, bg_color=bg_color)

        self.scenes: dict[str, tkscenes.Scene] = {}
        self._has_drawn = False

    def __getitem__(self, item: str) -> tkscenes.Scene:
        return self.scenes[item]

    def add_scene(self, key: str):
        """
        Add a scene
        :param key: Identifier
        :return:
        """
        self.scenes[key] = tkscenes.Scene()

    def change_scene(self, key: str):
        """
        Change the active scene
        :param key: The identifier of the scene you want to change to
        :return:
        """
        if self._has_drawn:
            for scene in self.scenes.keys():
                if scene == key:
                    continue

                self.scenes[scene].unload()

            self.scenes[key].load()

    def pack(self, _key=None, **kwargs):
        """
        Pack the main panel
        :param _key: The identifier of the scene which shows up first. If empty then it will show the first scene in the list.
        :param kwargs:
        :key padx: Add padding in x direction
        :key pady: Add padding in y direction
        :key ipadx: Add internal padding in x direction
        :key ipady: Add internal padding in x direction
        :return:
        """
        self._has_drawn = True

        if _key is None:
            _key = list(self.scenes.keys())[0]

        settings = {
            "padx": 0,
            "pady": 0,
            "ipadx": 0,
            "ipady": 0,
            "side": "top",
            "anchor": "center",
            "fill": None,
            "expand": False
        }

        for key in settings.keys():
            if key in kwargs.keys():
                settings[key] = kwargs[key]

        self.pane.pack(
            padx=settings["padx"],
            pady=settings["pady"],
            ipadx=settings["ipadx"],
            ipady=settings["ipady"],
            side=settings["side"],
            anchor=settings["anchor"],
            fill=settings["fill"],
            expand=settings["expand"]
        )

        self.scenes[_key].load()

    def pack_forget(self):
        """
        Unmap this widget
        :return:
        """
        self.pane.pack_forget()



from typing import Any
import customtkinter as ctk
import tkscenes

from .side_panel_child import SidePane as SidePane
from .side_panel_child import MainPane as MainPane


class CTkSidePanel(ctk.CTkBaseClass):
    """
    Create a side panel
    """
    def __init__(self, master: Any, **kwargs):
        super().__init__(master, **kwargs)

        self.frame = ctk.CTkFrame(master, fg_color="transparent")

        self.side_pane = SidePane(self.frame)
        self.main_pane = MainPane(self.frame, fg_color="transparent")

    def __getitem__(self, item: str) -> tkscenes.Scene:
        """
        Get the scene
        :param item: Identifier
        :return:
        """
        return self.main_pane[item]

    def add_scene(self, key: str, name=None, image=None, anchor="center"):
        """
        Add a scene
        :param key: Identifier
        :param name: Name of the entry
        :param image:
        :param anchor:
        :return:
        """
        if name is None:
            name = key

        self.side_pane.add(key, name, image, lambda: self.change_scene(key), anchor)
        self.main_pane.add_scene(key)

    def change_scene(self, key: str):
        """
        Change the current scene
        :param key: Identifier
        :return:
        """
        self.main_pane.change_scene(key)

    def _render_panes(self):
        self.side_pane.pack(fill="y", side="left", padx=5, pady=5)
        self.main_pane.pack(fill="both", side="left", expand=True, padx=5, pady=5)

    def pack(self, **kwargs):
        self.frame.pack(**kwargs)
        self._render_panes()

    def grid(self, **kwargs):
        self.frame.grid(**kwargs)
        self._render_panes()

    def place(self, **kwargs):
        self.frame.place(**kwargs)
        self._render_panes()

    def pack_forget(self):
        self.frame.pack_forget()

    def grid_forget(self):
        self.frame.grid_forget()

    def place_forget(self):
        self.frame.place_forget()

    @property
    def main_panel(self) -> ctk.CTkFrame:
        """
        :return: The main panel
        """
        return self.main_pane.pane



from typing import Any
import customtkinter as ctk
import datetime

from customtkinter import CTkFrame

from .calender import CTkCalender, month_names


class CTkDateSelect(ctk.CTkBaseClass):
    """
    Creates a dropdown for selecting dates
    """
    def __init__(self, master: Any, **kwargs):
        super().__init__(master, **kwargs)

        self._date = datetime.date.today()
        self._dropdown = ctk.CTkButton(master, text=str(self._date), command=self._open_popup)

        self._window = None

    def select_date(self, date: datetime.date):
        self._dropdown.configure(
            True,
            text=str(date)
        )
        self._window.destroy()

    def _open_popup(self):
        # stupid
        def insanity():
            self._calender[1].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 1)
                )
            )
            self._calender[2].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 2)
                )
            )
            self._calender[3].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 3)
                )
            )
            self._calender[4].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 4)
                )
            )
            self._calender[5].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 5)
                )
            )
            self._calender[6].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 6)
                )
            )
            self._calender[7].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 7)
                )
            )
            self._calender[8].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 8)
                )
            )
            self._calender[9].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 9)
                )
            )
            self._calender[10].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 10)
                )
            )
            self._calender[11].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 11)
                )
            )
            self._calender[12].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 12)
                )
            )
            self._calender[13].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 13)
                )
            )
            self._calender[14].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 14)
                )
            )
            self._calender[15].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 15)
                )
            )
            self._calender[16].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 16)
                )
            )
            self._calender[17].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 17)
                )
            )
            self._calender[18].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 18)
                )
            )
            self._calender[19].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 19)
                )
            )
            self._calender[20].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 20)
                )
            )
            self._calender[21].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 21)
                )
            )
            self._calender[22].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 22)
                )
            )
            self._calender[23].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 23)
                )
            )
            self._calender[24].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 24)
                )
            )
            self._calender[25].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 25)
                )
            )
            self._calender[26].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 26)
                )
            )
            self._calender[27].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 27)
                )
            )
            self._calender[28].configure(
                True,
                command=lambda: self.select_date(
                    datetime.date(self._calender.year, self._calender.month, 28)
                )
            )
            if self._calender.len_of_month >= 29:
                self._calender[29].configure(
                    True,
                    command=lambda: self.select_date(
                        datetime.date(self._calender.year, self._calender.month, 29)
                    )
                )
            if self._calender.len_of_month >= 30:
                self._calender[30].configure(
                    True,
                    command=lambda: self.select_date(
                        datetime.date(self._calender.year, self._calender.month, 30)
                    )
                )
            if self._calender.len_of_month >= 31:
                self._calender[31].configure(
                    True,
                    command=lambda: self.select_date(
                        datetime.date(self._calender.year, self._calender.month, 31)
                    )
                )

        # command
        def change_time(n: int):
            year = self._calender.year
            month = self._calender.month

            month += n

            if month > 12:
                month = 1
                year += 1
            if month < 1:
                month = 12
                year -= 1

            self._calender.change_time(
                datetime.date(year, month, 1)
            )

            insanity()

            self._text.configure(True, text=f"{month_names[month][:3]} {year}")

        # code
        self._window = ctk.CTkToplevel()
        self._window.title("Select a date")
        self._window.geometry("400x300")

        self._top_bar = CTkFrame(self._window)
        self._top_bar.pack(fill="x", padx=5, pady=5)

        self._top_bar.columnconfigure([0, 1, 2], weight=1)
        self._top_bar.rowconfigure(0, weight=1)

        self._calender = CTkCalender(self._window)
        self._calender.pack(fill="both", padx=5, pady=5, expand=True)

        insanity()

        self._text = ctk.CTkLabel(
            self._top_bar,
            text=f"{month_names[self._calender.month][:3]} {self._calender.year}"
        )
        self._next = ctk.CTkButton(
            self._top_bar, text="Next",
            command=lambda: change_time(1)
        )
        self._back = ctk.CTkButton(
            self._top_bar, text="Back",
            command=lambda: change_time(-1)
        )

        self._back.grid(row=0, column=0, padx=5, pady=5, sticky="news")
        self._text.grid(row=0, column=1, padx=5, pady=5)
        self._next.grid(row=0, column=2, padx=5, pady=5, sticky="news")

    def pack(self, **kwargs):
        self._dropdown.pack(**kwargs)

    def grid(self, **kwargs):
        self._dropdown.grid(**kwargs)

    def place(self, **kwargs):
        self._dropdown.place(**kwargs)



from typing import Any
import customtkinter as ctk
import datetime
import math
import tkscenes as scn

month_names = [
    None,
    "January",
    "February",
    "March",
    "April",
    "May",
    "June",
    "July",
    "August",
    "September",
    "October",
    "November",
    "December"
]

month_len = [None, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]


def is_leep_year(year: int) -> bool:
    return False


class CTkCalender(ctk.CTkBaseClass):
    def __init__(
            self, master: Any,
            date=datetime.date.today(),
            today_color=None,
            today_highlight_color=None,
            **kwargs
    ):
        super().__init__(master, **kwargs)

        self._today = datetime.date.today()

        self.frame = ctk.CTkFrame(master)
        self.frame.columnconfigure((0, 1, 2, 3, 4, 5, 6), weight=1)
        self.frame.rowconfigure(0, weight=1)
        self.labels = [
            ctk.CTkLabel(self.frame, text="Sun"),
            ctk.CTkLabel(self.frame, text="Mon"),
            ctk.CTkLabel(self.frame, text="Tue"),
            ctk.CTkLabel(self.frame, text="Wed"),
            ctk.CTkLabel(self.frame, text="Thu"),
            ctk.CTkLabel(self.frame, text="Fri"),
            ctk.CTkLabel(self.frame, text="Sat")
        ]
        self.dates = scn.Scene()

        if today_color is None:
            self._today_color = "#f07220"
        else:
            self._today_color = today_color

        if today_highlight_color is None:
            self._today_highlight_color = "#a8600e"
        else:
            self._today_highlight_color = today_highlight_color

        self._config_grid(date)
        self._add_dates()

        self._has_drawn = False

    def _add_dates(self):
        for i in range(self._num_of_days):
            if datetime.date(self._year, self._month, i + 1) == self._today:
                color = self._today_color
                hover_color = self._today_highlight_color
            else:
                color = None
                hover_color = None
            self.dates[str(i + 1)] = ctk.CTkButton(
                self.frame,
                text=str(i + 1),
                fg_color=color,
                hover_color=hover_color
            )
            self.dates[str(i + 1)].set_mode(
                "grid",
                row=(i + self._weekday) // 7 + 1,
                column=(i + self._weekday) % 7,
                padx=5, pady=5,
                sticky="nsew"
            )

    def _config_grid(self, date):
        self.frame.rowconfigure([0, 1, 2, 3, 4, 5], weight=0)

        self._month = date.month
        self._year = date.year
        self._weekday = datetime.date(self._year, self._month, 1).weekday() + 1

        if self._weekday > 6:
            self._weekday = 0

        if self._month == 2:
            if is_leep_year(self._year):
                self._num_of_days = 29
            else:
                self._num_of_days = 28
        else:
            self._num_of_days = month_len[self._month]

        self.frame.rowconfigure([int(i) + 1 for i in range(math.ceil((self._weekday + self._num_of_days) / 7))], weight=1)

    def __getitem__(self, item: str | int) -> ctk.CTkButton:
        return self.dates[str(item)].widget

    def _map(self):
        self._has_drawn = True

        # pack date bar
        for index, item in enumerate(self.labels):
            item.grid(row=0, column=index)

        self.dates.load()

    def pack(self, **kwargs):
        self.frame.pack(**kwargs)
        self._map()

    def grid(self, **kwargs):
        self.frame.pack(**kwargs)
        self._map()

    def place(self, **kwargs):
        self.frame.pack(**kwargs)
        self._map()

    def pack_forget(self):
        self.frame.pack_forget()

    def grid_forget(self):
        self.frame.grid_forget()

    def place_forget(self):
        self.frame.place_forget()

    def change_time(self, date: datetime.date) -> None:
        self.dates.destroy()
        self._config_grid(date)
        self._add_dates()

        if self._has_drawn:
            self.dates.load()

    @property
    def year(self) -> int:
        return self._year

    @property
    def month(self) -> int:
        return self._month

    @property
    def len_of_month(self) -> int:
        return len(self.dates.children)
